/**
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Wei&szlig;enfelser Stra&szlig;e 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.wfs.wfs;

/**
 * Legt die Typen, Attributgruppen und Aspekte fest, die &uuml;ber des WFS
 * bereitgestellt werden sollen.
 * Es werden "ObjektTyp", "Attributgruppe" und "Aspekt" der bereitzustellenden
 * Daten konfiguriert.
 * Mehrere Eintr&auml;ge f&uuml;r einen ObjektTyp sind mglich, Das Feature wird 
 * dann aus allen Attributgruppen dieses Typs gebildet.
 * Die die Geometrie eines Typs beschreibende Attributgruppe mu&szlig; nicht
 * notwendigerweise parametriert werden. Die SWE versucht fr alle konfigurierten
 * Typen eine Geometrie nach folgendem Algorithmus zu bestimmen:
 * 
 * <p>Wenn der Typ von einem der Typen des TeilmodellGeoReferenzierungGlobal
 * abgeleitet ist, wird die Geoemtrietyp aus dem entsprechenden Basistyp
 * gebildet. Konnte damit keine Geometrie gefunden werden, werden die f&uuml;r
 * den Typ die konfigurierten Attributgruppen nach folgenden Attributgruppen  
 * als Basis f&uuml;r die Geometrie durchsucht:
 * atg.punktKoordinaten
 * atg.linienKoordinaten
 * atg.flchenKoordinaten
 * atg.komplexKoordinaten
 * atg.bestehtAusLinienObjekten
 * 
 * Konnte eine derartige Attributgruppe nicht gefunden werden, wird der Typ auf
 * der Existenz einer der obigen Atributgruppen &uuml;berpr&uuml;ft und diese 
 * dann verwendet.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: ExportObjekte2.java 6068 2008-01-30 16:52:47Z gieseler $
 *
 */
public class ExportObjekte2 {
	/**
	 * Enth&aumllt je Eintrag jeweils die Pid des ObjektTyps, die Pid der 
	 * Attributgruppe sowie den Aspekt.
	 */
	@SuppressWarnings("nls")
	private static final Object[][] exportObjekte = new Object[][]{
		//  ObjektTyp, Attributgruppe, Aspekt
		{"typ.messQuerschnitt", "atg.punktKoordinaten", "asp.eigenschaften"}
	};


	/**
	 * @return die Liste der zu exportierenden Objekte
	 */
	public static Object[][] getExportObjekte(){
		return exportObjekte;
	}
}
