/*
 * Segment 2 (KEx),  SWE 2.WebFS Web Feature Service
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.wfs.wfs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.geotools.data.DataStore;
import org.geotools.data.vrz3.VRZ3DataStoreFactory;
import org.geotools.feature.FeatureType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Steuerung der Geoserver-Applikation.
 * 
 * Es werden Methoden zur Anpassung der Einstellungen sowie zum Starten und
 * Beenden der Anwendung bereitgestellt.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: GeoServerApplikation.java 8304 2008-04-16 10:19:12Z gieseler $
 * 
 */

public class GeoServerApplikation implements Runnable {

	/** Name der Geoserver-Anwendung. */
	private static final String GEOSERVER_BEZEICHNUNG = "VRZ 3 - WFS";

	/** Titel der Geoserver-Anwendung. */
	private static final String GEOSERVER_TITEL = "KEx Web Feature Service";

	/** Zugriffsbeschr&auml;nkungen der Geoserver-Anwendung. */
	private static final String GEOSERVER_ZUGRIFFSBESCHRAENKUNG = "Die Daten dieses Dienstes sind Eigentumdes Landes Baden-Wrttemberg. Sie bentigen zur Nutzung der Daten eine schriftliche Erlaubnis des Landes Baden-Wrttemberg";

	/** Geb&uuml;renangabe der Geoserver-Anwendung. */
	private static final String GEOSERVER_GEBUEHREN = "Die Verwendung der Daten ist geb&uuml;hrenpflichtig.";

	/** Pflegende Stelle der Geoserver-Anwendung. */
	private static final String GEOSERVER_DATENPFLEGE = "http://www.rpt.bwl.de";

	/** Schl&uuml;sselworte der Geoserver-Anwendung. */
	private static final String[] GEOSERVER_SCHLUESSELWORTE = { "VRZ 3", "WFS" };

	/** Kurzbeschreibung der Geoserver-Anwendung. */
	private static final String GEOSERVER_KURZBESCHREIBUNG = "Export von Anlagendaten aus der VRZ 3.";

	/** Hauptverzeichnis der Geoserver-Applikationsdateien. */
	public static String geoserverRoot = "./geoserver";

	/** Verzeichnis der Bin&auml;rdateien der Geoserver-Applikation. */
	private static final String GEOSERVER_BIN_DIRECTORY = "bin";

	/** Verzeichnis der Geoserver-Konfigurationsdateien. */
	private static final String GEOSERVER_CONFIGURATION_DIRECTORY = "etc";

	/** Verzeichnis der Geoserver-Konfigurationsdateien. */
	public static final String GEOSERVER_DATA_DIRECTORY = "data_dir";

	/** Verzeichnis der Geoserver-Featurekonfiguration. */
	public static final String GEOSERVER_FEATURES_DIRECTORY = "featureTypes";

	/**
	 * Dateiname der Geoserver-Konfigurationsdatei, welche die HTTP-Port
	 * Konfiguration enth&auml;lt.
	 */
	private static final String GEOSERVER_PORT_CONFIGURATION_FILE = "jetty.xml";

	/** Dateiname der Geoserver-Konfigurationsdatei. */
	private static final String GEOSERVER_SERVICE_FILE = "services.xml";

	/**
	 * Dateiname der Geoserver-Konfigurationsdatei, welche die DataStore
	 * Konfiguration enth&auml;lt.
	 */
	private static final String GEOSERVER_CATALOG_FILE = "catalog.xml";

	/** Der Name des Geoserver-Start-Skiptes unter UNIX. */
	private static final String GEOSERVER_UNIX_START_SCRIPT = "startup.sh";

	/** Der Name des Geoserver-Start-Skiptes unter WINDOWS. */
	private static final String GEOSERVER_WINDOWS_START_SCRIPT = "startup.bat";

	/** Die ID des VRZ3-Datastores in der Geoserver-Umgebung. */
	public static final String GEOSERVER_VRZ3_DATASTORE_ID = "VRZ3DS";

	/** Die ID des VRZ3-Datastores in der Geoserver-Umgebung. */
	public static final String GEOSERVER_SRS = "4326";

	/** DebugLogger fr Debug-Ausgaben. */
	private static final Debug DEBUG = Debug.getLogger();

	/** Der benutzte HTTP-Port. */
	private int httpPort;

	/** Der benutzte Namespace. */
	private String nameSpace = (String) VRZ3DataStoreFactory.PARAM_NAMESPACE.sample;

	/** Die benutzte Namespace-URI. */
	private final String namespaceUri = "http://www.bsvrz.de";

	/** Datenverteiler. */
	private String davHost;

	/** Datenverteiler-Authentifizierung. */
	private String davAuthentifizierung;

	/** Datenverteiler-Nutzer. */
	private String davBenutzer;

	/** Der laufende GEOSERVER-Prozess. */
	Process geoserverProcess;

	/** Dateiname der Spezifikationsdatei f&uuml;r die zu exportierenden Objekte. */
	private String exportObjekteDatei;

	/**
	 * Erzeugt eine Instanz mit dem Wurzelverzeichnis der Geoserver-Anwendung.
	 * 
	 * @param geoserverRootPfad
	 *            Wurzelverzeichnis der Geoserver-Anwendung
	 */
	public GeoServerApplikation(String geoserverRootPfad) {
		super();
		geoserverRoot = geoserverRootPfad;
	}

	/**
	 * Erzeugt die 'info.xml'-Datei des Features.
	 * 
	 * Struktur:<br>
	 * 
	 * <pre>
	 * &lt;featureType datastore = &quot;VRZ3DS&quot; &gt;
	 *  &lt;name&gt;InneresStrassenSegment&lt;/name&gt;
	 *  &lt;SRS&gt;4326&lt;/SRS&gt;
	 *  &lt;SRSHandling&gt;0&lt;/SRSHandling&gt; 
	 *  &lt;title&gt;InneresStrassenSegment_Type&lt;/title&gt;
	 *  &lt;abstract&gt;Erzeugt aus VRZ3-Konfiguration&lt;/abstract&gt; &lt;wmspath&gt;/&lt;/wmspath&gt;
	 *  &lt;numDecimals value = &quot;8&quot; /&gt; &lt;keywords&gt;InneresStrassenSegment VRZ3DS&lt;/keywords&gt;
	 *  &lt;latLonBoundingBox dynamic = &quot;false&quot; miny = &quot;47.0&quot; maxy = &quot;50.0&quot; maxx =
	 * &quot;11.0&quot; minx = &quot;7.0&quot; /&gt; &lt;nativeBBox dynamic = &quot;false&quot; miny = &quot;47.544567&quot;
	 * maxy = &quot;49.858315&quot; maxx = &quot;10.246678&quot; minx = &quot;7.522368999999999&quot; /&gt; 
	 *  &lt;styles default = &quot;ISS&quot; /&gt;
	 *  &lt;cacheinfo enabled = &quot;true&quot; maxage = &quot;3600&quot; /&gt; 
	 * &lt;/featureType&gt;
	 * </pre>
	 * 
	 * @param featuretyp
	 *            Feature-Typ
	 * 
	 * @param featuredir
	 *            Feature-Verzeichnis in der Geoserver-Umgebung
	 * 
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	private void erzeugeFeatureInfo(FeatureType featuretyp, File featuredir)
			throws WFSException {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			Document doc = factory.newDocumentBuilder().newDocument();

			Element root = doc.createElement("featureType");
			doc.appendChild(root);
			root.setAttribute("datastore",
					GeoServerApplikation.GEOSERVER_VRZ3_DATASTORE_ID);

			Element e = doc.createElement("name");
			e.setTextContent(featuretyp.getTypeName());
			root.appendChild(e);

			e = doc.createElement("SRS");
			e.setTextContent(GeoServerApplikation.GEOSERVER_SRS);
			root.appendChild(e);

			e = doc.createElement("SRSHandling");
			e.setTextContent("0");
			root.appendChild(e);

			e = doc.createElement("title");
			e.setTextContent(featuretyp.getTypeName() + "_Type");
			root.appendChild(e);

			e = doc.createElement("abstract");
			e.setTextContent(featuretyp.getTypeName()
					+ " der DAV-Konfiguration");
			root.appendChild(e);

			e = doc.createElement("wmspath");
			e.setTextContent("/");
			root.appendChild(e);

			e = doc.createElement("numDecimals");
			e.setAttribute("value", "8");
			root.appendChild(e);

			e = doc.createElement("keywords");
			e.setTextContent(featuretyp.getTypeName() + "  VRZ3");
			root.appendChild(e);

			// <latLonBoundingBox dynamic = "false" miny = "-90.0" maxy = "90.0"
			// maxx = "180.0" minx = "-180.0" />
			// <latLonBoundingBox dynamic = "false" miny = "47.0" maxy = "50.0"
			// maxx = "11.0" minx = "7.0" />
			e = doc.createElement("latLonBoundingBox");
			e.setAttribute("dynamic", "false");
			e.setAttribute("miny", "47.0");
			e.setAttribute("maxy", "50.0");
			e.setAttribute("maxx", "11.0");
			e.setAttribute("minx", "7.0");
			root.appendChild(e);

			// <nativeBBox dynamic = "false" miny = "47.544567" maxy =
			// "49.858315" maxx = "10.246678" minx = "7.522368999999999" />

			e = doc.createElement("styles");
			Class<?> gclass = featuretyp.getDefaultGeometry().getType();
			if (gclass.equals(Point.class)) {
				e.setAttribute("default", "point");
			} else if (gclass.equals(LineString.class)
					|| gclass.equals(MultiLineString.class)) {
				e.setAttribute("default", "line");
			} else if (gclass.equals(Polygon.class)
					|| gclass.equals(GeometryCollection.class)) {
				e.setAttribute("default", "line");
			} else {
				e.setAttribute("default", "point");
			}

			root.appendChild(e);

			e = doc.createElement("cacheinfo");
			e.setAttribute("enabled", "true");
			e.setAttribute("maxage", "3600");
			root.appendChild(e);

			File infofile = new File(featuredir, "info.xml");
			XMLTools.schreibeXMLDatei(doc, infofile);
		} catch (Exception e) {
			throw new WFSException(e.getMessage());
		}
	}

	/**
	 * Erzeugt die notwendigen Konfigurationen in der Geoserver-Umgebung, damit
	 * das Feature bereitgestellt wird.
	 * 
	 * @param featuretype
	 *            Feature-Typ
	 * 
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	private void erzeugeFeatureKonfiguration(FeatureType featuretype)
			throws WFSException {
		File featuredir = new File(getFeatureDirectory(),
				GEOSERVER_VRZ3_DATASTORE_ID + "_" + featuretype.getTypeName());

		try {
			if (!featuredir.exists()) {
				featuredir.mkdir();
			}

			erzeugeFeatureInfo(featuretype, featuredir);
			erzeugeTemplates(featuretype.getTypeName(), featuredir);
		} catch (Exception e) {
			throw new WFSException(e.getMessage());
		}
	}

	/**
	 * Erzeugt die Feature-Typ Dateien.
	 * 
	 * @param datastore
	 *            DataStore
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	public void erzeugeFeatureTypen(DataStore datastore) throws WFSException {
		try {
			loescheFeatureKonfiguration();
			for (String featuretype : datastore.getTypeNames()) {
				erzeugeFeatureKonfiguration(datastore.getSchema(featuretype));
			}
		} catch (Exception e) {
			throw new WFSException(e.getMessage());
		}
	}

	/**
	 * Erzeugt die notwendigen Template-Dateien.
	 * 
	 * 1. Datei: title.ftl - Die Datei legt fest, wie der Placemark-Name
	 * gebildet wird
	 * 
	 * @param typename
	 *            Feature-Typname
	 * @param featuredir
	 *            Feature-Verzeichnis in der Geoserver-Umgebung
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	private void erzeugeTemplates(String typename, File featuredir)
			throws WFSException {
		PrintWriter pw = null;

		try {
			File tfile = new File(featuredir, "title.ftl");
			pw = new PrintWriter(new FileOutputStream(tfile));
			pw.println("${Name.value}");
		} catch (FileNotFoundException e) {
			throw new WFSException(e.getMessage());
		} finally {
			if (pw != null) {
				pw.close();
			}
		}
	}

	/**
	 * Findet ein Elelement in einem Document.
	 * 
	 * @param doc
	 *            Dokument
	 * @param element
	 *            Name des Elements
	 * @param attribut
	 *            Attribut des Elements
	 * @param attributwert
	 *            Wert des Attributs des Elements
	 * @return das gefundene Element
	 */
	private Node findeElement(Document doc, String element, String attribut,
			String attributwert) {
		NodeList nl = doc.getElementsByTagName(element);

		if (attribut == null) {
			return nl.getLength() > 0 ? nl.item(0) : null;
		}

		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			NamedNodeMap attrs = n.getAttributes();

			Node fid = attrs.getNamedItem(attribut);
			if (fid.getNodeValue().equals(attributwert)) {
				return n;
			}
		}

		return null;
	}

	/**
	 * Findet ein Element unter einem Knoten.
	 * 
	 * @param node
	 *            Knoten
	 * @param element
	 *            Element
	 * @param attribut
	 *            Attribut
	 * @param attributwert
	 *            Wert des Attributes
	 * @return Knoten gefundener Knoten
	 */
	private Node findeElement(Node node, String element, String attribut,
			String attributwert) {
		NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			if (n.getNodeName().equals(element)) {
				if (attribut == null) {
					return n;
				}

				NamedNodeMap attrs = n.getAttributes();

				Node fid = attrs.getNamedItem(attribut);
				if (fid != null && fid.getNodeValue().equals(attributwert)) {
					return n;
				}
			}
		}

		return null;
	}

	/**
	 * Gibt den Wert eines Attributes eines Knotens zur&uuml;ck.
	 * 
	 * @param node
	 *            Knoten
	 * @param attr
	 *            Attribut
	 * @return Attributwert
	 */
	@SuppressWarnings("unused")
	private String getAttributWert(Node node, String attr) {
		if (node.hasAttributes()) {
			return ((Element) node).getAttribute(attr);
		}

		if (node.getNodeName().equals(attr)) {
			NodeList childs = node.getChildNodes();
			for (int i = 0; i < childs.getLength(); i++) {
				if (childs.item(i).getNodeType() == Node.TEXT_NODE) {
					return childs.item(i).getNodeValue();
				}
			}

		}

		NodeList childs = node.getChildNodes();
		for (int i = 0; i < childs.getLength(); i++) {
			if (childs.item(i).getNodeType() == Node.ELEMENT_NODE) {
				String ret = getAttributWert(childs.item(i), attr);
				if (ret != null) {
					return ret;
				}
			}
		}

		return null;
	}

	/**
	 * Erzeugt den Namen des Feature-Typ Verzeichnisses innerhalb der
	 * Geoserver-Umgebung.
	 * 
	 * @return Feature-Typ Verzeichnis
	 */
	private File getFeatureDirectory() {
		File temp = new File(geoserverRoot);
		temp = new File(temp, GeoServerApplikation.GEOSERVER_DATA_DIRECTORY);
		return new File(temp, GeoServerApplikation.GEOSERVER_FEATURES_DIRECTORY);
	}

	/**
	 * L&ouml;scht die bestehenden FeatureType-Konfigurationen.
	 */
	private void loescheFeatureKonfiguration() {
		File fdir = getFeatureDirectory();
		for (File f : fdir.listFiles()) {
			if (f.isDirectory()) {
				loescheVerzeichnis(f);
			}
		}
	}

	/**
	 * L&ouml;scht ein Verzeichnis einschlie&szlig;lich dessen Inhalt.
	 * 
	 * @param dir
	 *            Verzeichnis
	 */
	private void loescheVerzeichnis(File dir) {
		for (File f : dir.listFiles()) {
			if (f.isDirectory()) {
				loescheVerzeichnis(f);
			} else {
				f.delete();
			}
		}

		dir.delete();
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		// bilde Name des Startskriptes
		File temp = new File(geoserverRoot);
		temp = new File(temp, GEOSERVER_BIN_DIRECTORY); //$NON-NLS-1$

		File startscript = null;

		List<String> cmds = new ArrayList<String>();

		if (System.getProperty("os.name").toUpperCase().indexOf("WINDOWS") != -1) {
			startscript = new File(temp, GEOSERVER_WINDOWS_START_SCRIPT);
			cmds.add(startscript.toString());
		} else {
			// Unix ist !Windows !!!
			startscript = new File(GEOSERVER_UNIX_START_SCRIPT);
			cmds.add("/bin/sh");
			cmds.add(startscript.toString());
		}

		ProcessBuilder builder = new ProcessBuilder(cmds);
		builder.directory(temp);
		builder.redirectErrorStream(true);

		// Geoserver bentigt die folgenden Umgebungsvariablen: JAVA_HOME und
		// GEOSERVER_HOME
		Map<String, String> environment = builder.environment();
		// JAVA_HOME
		if (!environment.containsKey("JAVA_HOME")) {
			String jhome = System.getProperty("java.home");
			environment.put("JAVA_HOME", jhome);
		}

		// GEOSERVER_HOME
		if (!environment.containsKey("GEOSERVER_HOME")) {
			environment.put("GEOSERVER_HOME", geoserverRoot);
		}

		Thread debugreader;
		try {
			DEBUG.info("GeoServer wird gestartet (" + cmds + ")");
			geoserverProcess = builder.start();
			DEBUG.info("GeoServer luft (" + geoserverProcess + ")");

			debugreader = new Thread() {
				@Override
				public void run() {
					BufferedReader r = new BufferedReader(
							new InputStreamReader(geoserverProcess
									.getInputStream()));
					String currentLine = null;
					try {
						while ((currentLine = r.readLine()) != null) {
							DEBUG.info(currentLine);
						}
					} catch (IOException e) {
						// e.printStackTrace();
						DEBUG.info("Geoserver OutputReader IOException");
					}
				}
			};

			debugreader.start();

			while (true) {
				Thread.sleep(2000);
			}

		} catch (IOException e) {
			DEBUG.info("Geoserver IOException");
		} catch (InterruptedException e) {
			DEBUG.info("Geoserver unterbrochen");
		}

		DEBUG.info("Geoserver Process wird beendet");

		if (geoserverProcess != null) {
			geoserverProcess.destroy();
		}
	}

	/**
	 * Setzt den Wert eines Attributes eines Knotens.
	 * 
	 * @param node
	 *            Knoten
	 * @param attr
	 *            Attribut
	 * @param wert
	 *            zu setzender Wert des Attributes
	 */
	private void setAttributWert(Node node, String attr, String wert) {
		if (node.hasAttributes()) {
			((Element) node).setAttribute(attr, wert);
		}

		NodeList childs = node.getChildNodes();
		for (int i = 0; i < childs.getLength(); i++) {
			if (childs.item(i).getNodeType() == Node.ELEMENT_NODE) {
				setAttributWert(childs.item(i), attr, wert);
			}
		}
	}

	/**
	 * Setzt die Authentifizierung f&uuml;r den DaV.
	 * 
	 * @param authentifizierung
	 *            Authentifizierung
	 */
	public void setDavAuthentifizierung(String authentifizierung) {
		davAuthentifizierung = authentifizierung;
	}

	/**
	 * Setzt den Benutzer f&uuml;r den DaV.
	 * 
	 * @param benutzer
	 *            Benutzer am DaV
	 */
	public void setDavBenutzer(String benutzer) {
		davBenutzer = benutzer;
	}

	/**
	 * Setzt den DaV-Host.
	 * 
	 * @param host
	 *            DaV-Host
	 */
	public void setDavHost(String host) {
		davHost = host;
	}

	/**
	 * Setzt die Spezifikationsdatei f&uuml;r die zu exportierenden Objekte.
	 * 
	 * @param exportObjekteDatei
	 *            Dateiname
	 */
	public void setExportObjekteDatei(String exportObjekteDatei) {
		this.exportObjekteDatei = exportObjekteDatei;
	}

	/**
	 * Setzt den zu benutzenden Port.
	 * 
	 * @param port
	 *            Portnummer
	 */
	public void setHttpPort(int port) {
		httpPort = port;
	}

	/**
	 * Setzt den Namespace.
	 * 
	 * @param namespace
	 *            Namespace
	 */
	public void setNamespace(String namespace) {
		this.nameSpace = namespace;
	}

	/**
	 * Setzt die VRZ3DataStore-Parameter in der Geoserver-Umgebung.
	 * 
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	private void setzeGeoserverDataStoreParameter() throws WFSException {
		File temp = new File(geoserverRoot);
		temp = new File(temp, GEOSERVER_DATA_DIRECTORY); //$NON-NLS-1$
		File cfgfile = new File(temp, GEOSERVER_CATALOG_FILE);

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder parser = factory.newDocumentBuilder();
			InputStream is = new FileInputStream(cfgfile);

			Document doc = parser.parse(is);

			Node portelement = findeElement(doc, "datastore", "id",
					GEOSERVER_VRZ3_DATASTORE_ID);
			if (portelement == null) {
				throw new WFSException(
						"Die VRZ3-Datastore Konfiguration konnte nicht gefunden werden");
			}

			Node parametergruppe = findeElement(portelement,
					"connectionParams", null, null);

			Node parameterelement = findeElement(parametergruppe, "parameter",
					"name", VRZ3DataStoreFactory.PARAM_NAMESPACE.key);
			if (parameterelement == null) {
				throw new WFSException("Der VRZ3-Datastore Parameter '"
						+ VRZ3DataStoreFactory.PARAM_NAMESPACE.key
						+ "' konnte nicht gefunden werden");
			}
			setAttributWert(parameterelement, "value", nameSpace);

			parameterelement = findeElement(parametergruppe, "parameter",
					"name", VRZ3DataStoreFactory.PARAM_DAV.key);
			if (parameterelement == null) {
				throw new WFSException("Der VRZ3-Datastore Parameter '"
						+ VRZ3DataStoreFactory.PARAM_DAV.key
						+ "' konnte nicht gefunden werden");
			}
			setAttributWert(parameterelement, "value", davHost);

			parameterelement = findeElement(parametergruppe, "parameter",
					"name",
					VRZ3DataStoreFactory.PARAM_DAV_AUTHENTIFIZIERUNG.key);
			if (parameterelement == null) {
				throw new WFSException("Der VRZ3-Datastore Parameter '"
						+ VRZ3DataStoreFactory.PARAM_DAV_AUTHENTIFIZIERUNG.key
						+ "' konnte nicht gefunden werden");
			}
			setAttributWert(parameterelement, "value", davAuthentifizierung);

			parameterelement = findeElement(parametergruppe, "parameter",
					"name", VRZ3DataStoreFactory.PARAM_DAV_NUTZER.key);
			if (parameterelement == null) {
				throw new WFSException("Der VRZ3-Datastore Parameter '"
						+ VRZ3DataStoreFactory.PARAM_DAV_NUTZER.key
						+ "' konnte nicht gefunden werden");
			}
			setAttributWert(parameterelement, "value", davBenutzer);

			parameterelement = findeElement(parametergruppe, "parameter",
					"name", VRZ3DataStoreFactory.PARAM_EXPORTCLASS.key);
			if (parameterelement == null) {
				throw new WFSException("Der VRZ3-Datastore Parameter '"
						+ VRZ3DataStoreFactory.PARAM_EXPORTCLASS.key
						+ "' konnte nicht gefunden werden");
			}

			setAttributWert(parameterelement, "value", exportObjekteDatei);

			// aktualisiere Namespace
			parametergruppe = findeElement(doc, "namespaces", null, null);
			if (parametergruppe == null) {
				throw new WFSException(
						"Das Element 'namespaces' konnte nicht gefunden werden");
			}

			parameterelement = findeElement(parametergruppe, "namespace",
					"prefix", nameSpace);
			if (parameterelement == null) {
				Element nselement = doc.createElement("namespace");
				parametergruppe.appendChild(nselement);
				nselement.setAttribute("prefix", nameSpace);
				nselement.setAttribute("uri", namespaceUri);
			} else {
				setAttributWert(parameterelement, "uri", namespaceUri);
			}

			XMLTools.schreibeXMLDatei(doc, cfgfile);

		} catch (ParserConfigurationException e) {
			DEBUG.error(e.getMessage());
		} catch (FileNotFoundException e) {
			// _debug.error(e.getMessage());
			throw new WFSException("Die Datei '" + cfgfile.toString()
					+ "' konnte nicht gefunden werden");
		} catch (SAXException e) {
			DEBUG.error(e.getMessage());
		} catch (IOException e) {
			DEBUG.error(e.getMessage());
		}

	}

	/**
	 * Setzt die notwendigen Parameter fr die Geoserver-Laufzeitumgebung.
	 * 
	 * @throws WFSException
	 *             bei Ausnahmen
	 */
	public void setzeGeoserverParameter() throws WFSException {
		setzeGeoserverPort();
		setzeGeoserverServerParameter();
		setzeGeoserverDataStoreParameter();
	}

	/**
	 * Setzt die Portnummer f&uuml;r Geoserver.
	 * 
	 */
	private void setzeGeoserverPort() {
		File temp = new File(geoserverRoot);
		temp = new File(temp, GEOSERVER_CONFIGURATION_DIRECTORY); //$NON-NLS-1$
		File cfgfile = new File(temp, GEOSERVER_PORT_CONFIGURATION_FILE);

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder parser = factory.newDocumentBuilder();
			InputStream is = new FileInputStream(cfgfile);

			Document doc = parser.parse(is);
			Node portelement = findeElement(doc, "SystemProperty", "name",
					"jetty.port");

			setAttributWert(portelement, "default", (new Integer(httpPort))
					.toString());

			XMLTools.schreibeXMLDatei(doc, cfgfile);

		} catch (ParserConfigurationException e) {
			DEBUG.error(e.getMessage());
		} catch (FileNotFoundException e) {
			// _debug.error(e.getMessage());
			DEBUG.error("Die Datei '" + cfgfile.toString()
					+ "' konnte nicht gefunden werden");
		} catch (SAXException e) {
			DEBUG.error(e.getMessage());
		} catch (IOException e) {
			DEBUG.error(e.getMessage());
		} catch (WFSException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Setzt die Server-Parameter in der Geoserver-Umgebung Datei.
	 * 
	 * @throws WFSException
	 *             bei Ausnahmen
	 * 
	 */
	private void setzeGeoserverServerParameter() throws WFSException {
		File temp = new File(geoserverRoot);
		temp = new File(temp, GEOSERVER_DATA_DIRECTORY); //$NON-NLS-1$
		File cfgfile = new File(temp, GEOSERVER_SERVICE_FILE);
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder parser = factory.newDocumentBuilder();
			InputStream is = new FileInputStream(cfgfile);

			Document doc = parser.parse(is);

			Node serviceelement = findeElement(doc, "service", "type", "WFS");
			if (serviceelement == null) {
				throw new WFSException(
						"Die WFS-Konfiguration konnte nicht gefunden werden");
			}

			Node parameterelement = findeElement(serviceelement, "name", null,
					null);
			parameterelement.setTextContent(GEOSERVER_BEZEICHNUNG);

			parameterelement = findeElement(serviceelement, "title", null, null);
			parameterelement.setTextContent(GEOSERVER_TITEL);

			parameterelement = findeElement(serviceelement, "abstract", null,
					null);
			parameterelement.setTextContent(GEOSERVER_KURZBESCHREIBUNG);

			parameterelement = findeElement(serviceelement, "onlineResource",
					null, null);
			parameterelement.setTextContent(GEOSERVER_DATENPFLEGE);

			parameterelement = findeElement(serviceelement, "maintainer", null,
					null);
			parameterelement.setTextContent(GEOSERVER_DATENPFLEGE);

			parameterelement = findeElement(serviceelement, "fees", null, null);
			parameterelement.setTextContent(GEOSERVER_GEBUEHREN);

			parameterelement = findeElement(serviceelement,
					"accessConstraints", null, null);
			parameterelement.setTextContent(GEOSERVER_ZUGRIFFSBESCHRAENKUNG);

			parameterelement = findeElement(serviceelement, "keywords", null,
					null);
			if (parameterelement != null) {
				// Liste lschen
				while (parameterelement.hasChildNodes()) {
					parameterelement.removeChild(parameterelement
							.getFirstChild());
				}

				for (int i = 0; i < GEOSERVER_SCHLUESSELWORTE.length; i++) {
					Element kelement = doc.createElement("keyword");
					kelement.setTextContent(GEOSERVER_SCHLUESSELWORTE[i]);
					parameterelement.appendChild(kelement);
				}
			}

			XMLTools.schreibeXMLDatei(doc, cfgfile);
		} catch (ParserConfigurationException e) {
			DEBUG.error(e.getMessage());
		} catch (FileNotFoundException e) {
			// _debug.error(e.getMessage());
			throw new WFSException("Die Datei '" + cfgfile.toString()
					+ "' konnte nicht gefunden werden");
		} catch (SAXException e) {
			DEBUG.error(e.getMessage());
		} catch (IOException e) {
			DEBUG.error(e.getMessage());
		}
	}
}
