/*
 * Segment 2 (KEx),  SWE 2.WebFS Web Feature Service
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weienfelser Strae 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package org.geotools.data.vrz3;

import java.io.IOException;
import java.net.URI;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.geotools.data.DataStore;
import org.geotools.data.DataStoreFactorySpi;

import de.bsvrz.kex.wfs.wfs.WFSException;

/**
 * DataStoreFactory-Implementation f&uuml;r den VRZ3DataStore.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: VRZ3DataStoreFactory.java 8819 2008-05-09 14:58:18Z gieseler $
 * 
 */

public class VRZ3DataStoreFactory implements DataStoreFactorySpi {

	/**
	 * DataStore - spezifische Parameter.
	 */
	/** Datenverteiler-Verbindung. */
	public static final Param PARAM_DAV = new Param("datenverteiler",
			String.class, "Datenverteiler", true, "127.0.0.1:8083");

	/** Datenverteiler-Benutzer. */
	public static final Param PARAM_DAV_NUTZER = new Param("benutzer",
			String.class, "Datenverteiler-Benutzer", true, "Tester");

	/** Datenverteiler-Authentifizierung. */
	public static final Param PARAM_DAV_AUTHENTIFIZIERUNG = new Param(
			"authentifizierung", String.class,
			"Datenverteiler-Authentifizierung", true, "geheim");

	/** Namespace. */
	public static final Param PARAM_NAMESPACE = new Param("namespace",
			URI.class, "Namespace", true, "vrz3wfs");

	/** Definition der zu exportiedenden Objekte. */
	public static final Param PARAM_EXPORTCLASS = new Param("exportobjekte",
			String.class, "Klasse Exportobjekte", false,
			"de.bsvrz.kex.wfs.wfs.ExportObjekteDefault");

	/**
	 * Erzeugt ein neues VRZ3DataStoreFactory Objekt.
	 */
	public VRZ3DataStoreFactory() {
		super();
	}

	/**
	 * Setzt die DataStore-Parameter aus dem &uuml;bergebenen Parametersatz.
	 * 
	 * @param param
	 *            zu setzender Parameter
	 * @param params
	 *            &uuml;bergebener Parametersatz
	 * @param map
	 *            DataStore-Parameter
	 * @param defa
	 *            Default-Wert
	 */
	private void addParamToMap(final Param param, final Map<?, ?> params,
			HashMap<String, Object> map, final Object defa) {
		Object val = null;

		try {
			val = param.lookUp(params);
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (val != null) {
			map.put(param.key, val);
		} else if (defa != null) {
			map.put(param.key, defa);
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#canProcess(java.util.Map)
	 */
	public boolean canProcess(Map params) {
		try {
			return processParams(params);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#createDataStore(java.util.Map)
	 */
	public DataStore createDataStore(Map parameter) throws IOException {
		if (!processParams(parameter)) {
			throw new IOException("The parameters map isn't correct.");
		}

		try {
			HashMap<String, Object> datastoreparameter = new HashMap<String, Object>();

			addParamToMap(PARAM_NAMESPACE, parameter, datastoreparameter, null);
			addParamToMap(PARAM_DAV, parameter, datastoreparameter, null);
			addParamToMap(PARAM_DAV_AUTHENTIFIZIERUNG, parameter,
					datastoreparameter, null);
			addParamToMap(PARAM_DAV_NUTZER, parameter, datastoreparameter, null);
			addParamToMap(PARAM_EXPORTCLASS, parameter, datastoreparameter,
					null);

			return new VRZ3DataStore(datastoreparameter);
		} catch (WFSException ex) {
			throw new IOException(ex.getMessage());
		} catch (Exception ex) {
			throw new IOException(ex.toString());
		}
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#createNewDataStore(java.util.Map)
	 */
	public DataStore createNewDataStore(Map params) throws IOException {
		return createDataStore(params); // return null????
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#getDescription()
	 */
	public String getDescription() {
		return "VRZ3-DataStore";
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#getDisplayName()
	 */
	public String getDisplayName() {
		return "VRZ3DataStore";
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.factory.Factory#getImplementationHints()
	 */
	public Map<?, ?> getImplementationHints() {
		return Collections.EMPTY_MAP;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#getParametersInfo()
	 */
	public Param[] getParametersInfo() {
		Param[] params = { PARAM_DAV, PARAM_DAV_NUTZER,
				PARAM_DAV_AUTHENTIFIZIERUNG, PARAM_EXPORTCLASS };

		return params;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.geotools.data.DataStoreFactorySpi#isAvailable()
	 */
	public boolean isAvailable() {
		return true;
	}

	/**
	 * Zusatzfunktion zur Prameterverarbeitung.
	 * 
	 * @param parameter
	 *            Parameter
	 * @return true bei erfolgreicher Bearbeitung, sonst false
	 */
	private boolean processParams(Map<?, ?> parameter) {
		return true;
	}
}
